@@ -141,7 +141,7 @@ extern const char* TCN_UNKNOWN_AUTH_METHOD;
 #endif // SSL_OP_NO_TLSv1_3
 
 /* OpenSSL 1.0.2 compatibility */
-#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
+#if OPENSSL_VERSION_NUMBER < 0x10100000L || (defined(LIBRESSL_VERSION_NUMBER) && LIBRESSL_VERSION_NUMBER < 0x20700000)
 #define TLS_method SSLv23_method
 #define TLS_client_method SSLv23_client_method
 #define TLS_server_method SSLv23_server_method
@@ -183,6 +183,67 @@ extern void *SSL_temp_keys[SSL_TMP_KEY_MAX];
 #define TCN_X509_V_ERR_UNSPECIFIED (X509_V_ERR_UNSPECIFIED)
 #endif /*X509_V_ERR_UNSPECIFIED*/
 
+// BoringSSL compat
+#ifndef OPENSSL_IS_BORINGSSL
+#ifndef SSL_ERROR_WANT_PRIVATE_KEY_OPERATION
+#define SSL_ERROR_WANT_PRIVATE_KEY_OPERATION -1
+#endif // SSL_ERROR_WANT_PRIVATE_KEY_OPERATION
+
+// SSL_SIGN_* are signature algorithm values as defined in TLS 1.3.
+#ifndef SSL_SIGN_RSA_PKCS1_SHA1
+#define SSL_SIGN_RSA_PKCS1_SHA1 0x0201
+#endif // SSL_SIGN_RSA_PKCS1_SHA1
+
+#ifndef SSL_SIGN_RSA_PKCS1_SHA256
+#define SSL_SIGN_RSA_PKCS1_SHA256 0x0401
+#endif // SSL_SIGN_RSA_PKCS1_SHA256
+
+#ifndef SSL_SIGN_RSA_PKCS1_SHA384
+#define SSL_SIGN_RSA_PKCS1_SHA384 0x0501
+#endif // SSL_SIGN_RSA_PKCS1_SHA384
+
+#ifndef SSL_SIGN_RSA_PKCS1_SHA512
+#define SSL_SIGN_RSA_PKCS1_SHA512 0x0601
+#endif // SSL_SIGN_RSA_PKCS1_SHA512
+
+#ifndef SSL_SIGN_ECDSA_SHA1
+#define SSL_SIGN_ECDSA_SHA1 0x0203
+#endif // SSL_SIGN_ECDSA_SHA1
+
+#ifndef SSL_SIGN_ECDSA_SECP256R1_SHA256
+#define SSL_SIGN_ECDSA_SECP256R1_SHA256 0x0403
+#endif // SSL_SIGN_ECDSA_SECP256R1_SHA256
+
+#ifndef SSL_SIGN_ECDSA_SECP384R1_SHA384
+#define SSL_SIGN_ECDSA_SECP384R1_SHA384 0x0503
+#endif // SSL_SIGN_ECDSA_SECP384R1_SHA384
+
+#ifndef SSL_SIGN_ECDSA_SECP521R1_SHA512
+#define SSL_SIGN_ECDSA_SECP521R1_SHA512 0x0603
+#endif
+
+#ifndef SSL_SIGN_RSA_PSS_RSAE_SHA256
+#define SSL_SIGN_RSA_PSS_RSAE_SHA256 0x0804
+#endif // SSL_SIGN_RSA_PSS_RSAE_SHA256
+
+#ifndef SSL_SIGN_RSA_PSS_RSAE_SHA384
+#define SSL_SIGN_RSA_PSS_RSAE_SHA384 0x0805
+#endif // SSL_SIGN_RSA_PSS_RSAE_SHA384
+
+#ifndef SSL_SIGN_RSA_PSS_RSAE_SHA512
+#define SSL_SIGN_RSA_PSS_RSAE_SHA512 0x0806
+#endif // SSL_SIGN_RSA_PSS_RSAE_SHA512
+
+#ifndef SSL_SIGN_ED25519
+#define SSL_SIGN_ED25519 0x0807
+#endif // SSL_SIGN_ED25519
+
+#ifndef SSL_SIGN_RSA_PKCS1_MD5_SHA1
+#define SSL_SIGN_RSA_PKCS1_MD5_SHA1 0xff01
+#endif // SSL_SIGN_RSA_PKCS1_MD5_SHA1
+
+#endif // OPENSSL_IS_BORINGSSL
+
 // OCSP stapling should be present in OpenSSL as of version 1.0.0 but
 // we've only tested 1.0.2 and we need to support 1.0.1 because the
 // dynamically linked version of netty-tcnative is built with 1.0.1.
@@ -199,6 +260,11 @@ extern void *SSL_temp_keys[SSL_TMP_KEY_MAX];
 #define SSL_CTRL_GET_CLIENT_CERT_TYPES          103
 #endif
 
+#ifndef SSL_ERROR_WANT_CERTIFICATE_VERIFY
+// See https://github.com/google/boringssl/blob/chromium-stable/include/openssl/ssl.h#L538
+#define SSL_ERROR_WANT_CERTIFICATE_VERIFY       -1
+#endif
+
 typedef struct tcn_ssl_ctxt_t tcn_ssl_ctxt_t;
 
 typedef struct {
@@ -212,6 +278,10 @@ typedef struct {
     int verify_mode;
 } tcn_ssl_verify_config_t;
 
+#ifdef OPENSSL_IS_BORINGSSL
+extern const SSL_PRIVATE_KEY_METHOD private_key_method;
+#endif // OPENSSL_IS_BORINGSSL
+
 struct tcn_ssl_ctxt_t {
     apr_pool_t*              pool;
     SSL_CTX*                 ctx;
@@ -239,6 +309,12 @@ struct tcn_ssl_ctxt_t {
     jobject                  sni_hostname_matcher;
     jmethodID                sni_hostname_matcher_method;
 
+#ifdef OPENSSL_IS_BORINGSSL
+    jobject                  ssl_private_key_method;
+    jmethodID                ssl_private_key_sign_method;
+    jmethodID                ssl_private_key_decrypt_method;
+#endif // OPENSSL_IS_BORINGSSL
+
     tcn_ssl_verify_config_t  verify_config;
 
     int                      protocol;
@@ -266,22 +342,46 @@ struct tcn_ssl_ctxt_t {
     apr_uint32_t             ticket_keys_fail;
 
     unsigned char            context_id[SHA_DIGEST_LENGTH];
+
+    int                      use_tasks;
 };
 
+// Store the callback to run and also if it was consumed via SSL.getTask(...).
+typedef struct tcn_ssl_task_t tcn_ssl_task_t;
+struct tcn_ssl_task_t {
+    jboolean consumed;
+    jobject task;
+};
+
+tcn_ssl_task_t* tcn_ssl_task_new(JNIEnv*, jobject);
+void tcn_ssl_task_free(JNIEnv*, tcn_ssl_task_t*);
+
+typedef struct tcn_ssl_state_t tcn_ssl_state_t;
+struct tcn_ssl_state_t {
+    int handshakeCount;
+    tcn_ssl_ctxt_t *ctx;
+    tcn_ssl_task_t* ssl_task;
+    tcn_ssl_verify_config_t* verify_config;
+};
+
+#define TCN_GET_SSL_CTX(ssl, C)                             \
+    TCN_BEGIN_MACRO                                         \
+        tcn_ssl_state_t* _S = tcn_SSL_get_app_state(ssl);   \
+        if (_S == NULL) {                                   \
+            C = NULL;                                       \
+        } else {                                            \
+            C = _S->ctx;                                    \
+        }                                                   \
+    TCN_END_MACRO
+
 /*
  *  Additional Functions
  */
-void        tcn_SSL_init_app_data_idx(void);
+void        tcn_SSL_init_app_state_idx(void);
 // The app_data2 is used to store the tcn_ssl_ctxt_t pointer for the SSL instance.
-void       *tcn_SSL_get_app_data2(SSL *);
-void        tcn_SSL_set_app_data2(SSL *, void *);
-// The app_data3 is used to store the handshakeCount pointer for the SSL instance.
-void       *tcn_SSL_get_app_data3(SSL *);
-void        tcn_SSL_set_app_data3(SSL *, void *);
-// The app_data4 is used to store the tcn_ssl_verify_config_t pointer for the SSL instance.
-// This will initially point back to the tcn_ssl_ctxt_t in tcn_ssl_ctxt_t.
-void       *tcn_SSL_get_app_data4(SSL *);
-void        tcn_SSL_set_app_data4(SSL *, void *);
+void       *tcn_SSL_get_app_state(const SSL *);
+void        tcn_SSL_set_app_state(SSL *, void *);
+
 int         tcn_SSL_password_callback(char *, int, int, void *);
 DH         *tcn_SSL_dh_get_tmp_param(int);
 DH         *tcn_SSL_callback_tmp_DH(SSL *, int, int);
@@ -294,8 +394,12 @@ DH         *tcn_SSL_callback_tmp_DH_4096(SSL *, int, int);
 int         tcn_SSL_CTX_use_certificate_chain(SSL_CTX *, const char *, bool);
 int         tcn_SSL_CTX_use_certificate_chain_bio(SSL_CTX *, BIO *, bool);
 int         tcn_SSL_CTX_use_client_CA_bio(SSL_CTX *, BIO *);
-int         tcn_SSL_use_certificate_chain_bio(SSL *, BIO *, bool);
+
+#ifndef OPENSSL_IS_BORINGSSL
 X509        *tcn_load_pem_cert_bio(const char *, const BIO *);
+int         tcn_SSL_use_certificate_chain_bio(SSL *, BIO *, bool);
+#endif // OPENSSL_IS_BORINGSSL
+
 EVP_PKEY    *tcn_load_pem_key_bio(const char *, const BIO *);
 int         tcn_set_verify_config(tcn_ssl_verify_config_t* c, jint tcn_mode, jint depth);
 int         tcn_EVP_PKEY_up_ref(EVP_PKEY* pkey);
@@ -305,16 +409,24 @@ int         tcn_SSL_callback_select_next_proto(SSL *, unsigned char **, unsigned
 int         tcn_SSL_callback_alpn_select_proto(SSL *, const unsigned char **, unsigned char *, const unsigned char *, unsigned int, void *);
 const char *tcn_SSL_cipher_authentication_method(const SSL_CIPHER *);
 
-#if defined(OPENSSL_IS_BORINGSSL) || (OPENSSL_VERSION_NUMBER >= 0x10002000L && !defined(LIBRESSL_VERSION_NUMBER))
+#ifdef OPENSSL_IS_BORINGSSL
+enum ssl_verify_result_t tcn_SSL_cert_custom_verify(SSL* ssl, uint8_t *out_alert);
+#endif // OPENSSL_IS_BORINGSSL
+
+#if (OPENSSL_VERSION_NUMBER >= 0x10002000L && !defined(LIBRESSL_VERSION_NUMBER)) || LIBRESSL_VERSION_NUMBER >= 0x2090200fL
+
+#ifndef OPENSSL_IS_BORINGSSL
 #define tcn_SSL_add1_chain_cert(ssl, x509) SSL_add1_chain_cert(ssl, x509)
 #define tcn_SSL_add0_chain_cert(ssl, x509) SSL_add0_chain_cert(ssl, x509)
+#endif // OPENSSL_IS_BORINGSSL
+
 #define tcn_SSL_get0_certificate_types(ssl, clist) SSL_get0_certificate_types(ssl, clist)
 #else
 // This is what is defined in the SSL_add1_chain_cert / SSL_add0_chain_cert / SSL_get0_certificate_types macros.
 #define tcn_SSL_add1_chain_cert(ssl, x509) SSL_ctrl(ssl, SSL_CTRL_CHAIN_CERT, 1, (char *) x509)
 #define tcn_SSL_add0_chain_cert(ssl, x509) SSL_ctrl(ssl, SSL_CTRL_CHAIN_CERT, 0, (char *) x509)
 #define tcn_SSL_get0_certificate_types(ssl, clist) SSL_ctrl(ssl, SSL_CTRL_GET_CLIENT_CERT_TYPES, 0, (char *)(clist))
-#endif
+#endif // defined(OPENSSL_IS_BORINGSSL) || (OPENSSL_VERSION_NUMBER >= 0x10002000L && !defined(LIBRESSL_VERSION_NUMBER))
 
 #if defined(__GNUC__) || defined(__GNUG__)
     // only supported with GCC, this will be used to support different openssl versions at the same time.
@@ -326,9 +438,11 @@ const char *tcn_SSL_cipher_authentication_method(const SSL_CIPHER *);
     extern void SSL_get0_alpn_selected(const SSL *ssl, const unsigned char **data,
            unsigned *len) __attribute__((weak));
 
+#ifndef OPENSSL_IS_BORINGSSL
     extern X509_VERIFY_PARAM *SSL_get0_param(SSL *ssl) __attribute__((weak));
     extern void X509_VERIFY_PARAM_set_hostflags(X509_VERIFY_PARAM *param, unsigned int flags) __attribute__((weak));
     extern int X509_VERIFY_PARAM_set1_host(X509_VERIFY_PARAM *param, const char *name, size_t namelen) __attribute__((weak));
+#endif // OPENSSL_IS_BORINGSSL
 
     extern int SSL_get_sigalgs(SSL *s, int idx, int *psign, int *phash, int *psignhash, unsigned char *rsig, unsigned char *rhash) __attribute__((weak));
     extern void SSL_CTX_set_cert_cb(SSL_CTX *c, int (*cert_cb)(SSL *ssl, void *arg), void *arg) __attribute__((weak));
