@@ -80,14 +80,17 @@ const char* TCN_UNKNOWN_AUTH_METHOD = "UNKNOWN";
 const char* tcn_SSL_cipher_authentication_method(const SSL_CIPHER* cipher){
 #ifdef OPENSSL_IS_BORINGSSL
 	return SSL_CIPHER_get_kx_name(cipher);
-#elif OPENSSL_VERSION_NUMBER >= 0x10100000L && !defined(LIBRESSL_VERSION_NUMBER)
+#elif OPENSSL_VERSION_NUMBER >= 0x10100000L && (!defined(LIBRESSL_VERSION_NUMBER) || LIBRESSL_VERSION_NUMBER >= 0x2070000fL)
     switch (SSL_CIPHER_get_kx_nid(cipher)) {
         case NID_kx_rsa:
             return SSL_TXT_RSA;
         case NID_kx_dhe:
             switch (SSL_CIPHER_get_auth_nid(cipher)) {
+#ifndef LIBRESSL_VERSION_NUMBER
                 case NID_auth_dss:
                     return "DHE_" SSL_TXT_DSS;
+#endif // LIBRESSL_VERSION_NUMBER
+
                 case NID_auth_rsa:
                     return "DHE_" SSL_TXT_RSA;
                 case NID_auth_null:
@@ -111,7 +114,7 @@ const char* tcn_SSL_cipher_authentication_method(const SSL_CIPHER* cipher){
             // Let us just pick one as we could use whatever we want.
             // See https://www.openssl.org/docs/man1.1.1/man3/SSL_CIPHER_get_kx_nid.html
             return "ECDHE_" SSL_TXT_RSA;
-#endif
+#endif // OPENSSL_NO_TLS1_3
         default:
             return TCN_UNKNOWN_AUTH_METHOD;
     }
@@ -164,60 +167,31 @@ const char* tcn_SSL_cipher_authentication_method(const SSL_CIPHER* cipher){
  * also note that OpenSSL increments at static variable when
  * SSL_get_ex_new_index() is called, so we _must_ do this at startup.
  */
-static int tcn_SSL_app_data2_idx = -1;
-static int tcn_SSL_app_data3_idx = -1;
-static int tcn_SSL_app_data4_idx = -1;
-void tcn_SSL_init_app_data_idx()
+static int tcn_SSL_app_state_idx = -1;
+
+void tcn_SSL_init_app_state_idx()
 {
     int i;
 
-    if (tcn_SSL_app_data2_idx == -1) {
+    if (tcn_SSL_app_state_idx == -1) {
         /* we _do_ need to call this two times */
         for (i = 0; i <= 1; i++) {
-            tcn_SSL_app_data2_idx = SSL_get_ex_new_index(0, "tcn_ssl_ctxt_t*", NULL, NULL, NULL);
+            tcn_SSL_app_state_idx = SSL_get_ex_new_index(0, "tcn_ssl_state_t*", NULL, NULL, NULL);
         }
     }
-
-    if (tcn_SSL_app_data3_idx == -1) {
-        tcn_SSL_app_data3_idx = SSL_get_ex_new_index(0, "int* handshakeCount", NULL, NULL, NULL);
-    }
-
-    if (tcn_SSL_app_data4_idx == -1) {
-        tcn_SSL_app_data4_idx = SSL_get_ex_new_index(0, "tcn_ssl_verify_config_t*", NULL, NULL, NULL);
-    }
 }
 
-void *tcn_SSL_get_app_data2(SSL *ssl)
+void *tcn_SSL_get_app_state(const SSL *ssl)
 {
-    return (void *)SSL_get_ex_data(ssl, tcn_SSL_app_data2_idx);
+    return (void *)SSL_get_ex_data(ssl, tcn_SSL_app_state_idx);
 }
 
-void tcn_SSL_set_app_data2(SSL *ssl, void *arg)
+void tcn_SSL_set_app_state(SSL *ssl, void *arg)
 {
-    SSL_set_ex_data(ssl, tcn_SSL_app_data2_idx, (char *)arg);
+    SSL_set_ex_data(ssl, tcn_SSL_app_state_idx, (char *)arg);
     return;
 }
 
-void *tcn_SSL_get_app_data3(SSL *ssl)
-{
-    return SSL_get_ex_data(ssl, tcn_SSL_app_data3_idx);
-}
-
-void tcn_SSL_set_app_data3(SSL *ssl, void *arg)
-{
-    SSL_set_ex_data(ssl, tcn_SSL_app_data3_idx, arg);
-}
-
-void *tcn_SSL_get_app_data4(SSL *ssl)
-{
-    return SSL_get_ex_data(ssl, tcn_SSL_app_data4_idx);
-}
-
-void tcn_SSL_set_app_data4(SSL *ssl, void *arg)
-{
-    SSL_set_ex_data(ssl, tcn_SSL_app_data4_idx, arg);
-}
-
 int tcn_SSL_password_callback(char *buf, int bufsiz, int verify,
                           void *cb)
 {
@@ -362,7 +336,7 @@ static DH *get_dh(int idx)
         goto error;
     } else {
 // DH_set0_pqg() was introduced to initialize the DH parameters in OpenSSL 1.1.0 and DH is opaque now.
-#if OPENSSL_VERSION_NUMBER >= 0x10100000L && !defined(LIBRESSL_VERSION_NUMBER)
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L && (!defined(LIBRESSL_VERSION_NUMBER) || LIBRESSL_VERSION_NUMBER >= 0x20700000)
         if (DH_set0_pqg(dh, p, NULL, g) == 0) {
             goto error;
         }
@@ -382,7 +356,7 @@ error:
 
 DH *tcn_SSL_dh_get_tmp_param(int key_len)
 {
-    DH *dh;
+    DH *dh = NULL;
 
     if (key_len == 512)
         dh = get_dh(SSL_TMP_KEY_DH_512);
@@ -448,7 +422,7 @@ DH *tcn_SSL_callback_tmp_DH_4096(SSL *ssl, int export, int keylen)
  */
 int tcn_SSL_CTX_use_certificate_chain(SSL_CTX *ctx, const char *file, bool skipfirst)
 {
-    BIO *bio;
+    BIO *bio = NULL;
     int n;
 
     if ((bio = BIO_new(BIO_s_file())) == NULL)
@@ -462,11 +436,19 @@ int tcn_SSL_CTX_use_certificate_chain(SSL_CTX *ctx, const char *file, bool skipf
     return n;
 }
 
+// TODO: in the future we may want to add a function which does not need X509 at all for this.
 static int SSL_CTX_setup_certs(SSL_CTX *ctx, BIO *bio, bool skipfirst, bool ca)
 {
-    X509 *x509;
+#ifdef OPENSSL_IS_BORINGSSL
+    STACK_OF(CRYPTO_BUFFER) *names = sk_CRYPTO_BUFFER_new_null();
+    CRYPTO_BUFFER *buffer = NULL;
+    uint8_t *outp = NULL;
+    int len;
+#endif // OPENSSL_IS_BORINGSSL
+
+    X509 *x509 = NULL;
     unsigned long err;
-    int n;
+    int n = 0;
 
     /* optionally skip a leading server certificate */
     if (skipfirst) {
@@ -476,9 +458,26 @@ static int SSL_CTX_setup_certs(SSL_CTX *ctx, BIO *bio, bool skipfirst, bool ca)
         X509_free(x509);
     }
 
-    n = 0;
     if (ca) {
         while ((x509 = PEM_read_bio_X509(bio, NULL, NULL, NULL)) != NULL) {
+#ifdef OPENSSL_IS_BORINGSSL
+            len = i2d_X509_NAME(X509_get_subject_name(x509), &outp);
+            if (len < 0) {
+                sk_CRYPTO_BUFFER_pop_free(names, CRYPTO_BUFFER_free);
+                X509_free(x509);
+                return -1;
+            }
+            buffer = CRYPTO_BUFFER_new(outp, len, NULL);
+            OPENSSL_free(outp);
+            outp = NULL;
+
+            if (buffer == NULL || sk_CRYPTO_BUFFER_push(names, buffer) <= 0) {
+                sk_CRYPTO_BUFFER_pop_free(names, CRYPTO_BUFFER_free);
+                CRYPTO_BUFFER_free(buffer);
+                X509_free(x509);
+                return -1;
+            }
+#else
             if (SSL_CTX_add_client_CA(ctx, x509) != 1) {
                 X509_free(x509);
                 return -1;
@@ -486,10 +485,21 @@ static int SSL_CTX_setup_certs(SSL_CTX *ctx, BIO *bio, bool skipfirst, bool ca)
             // SSL_CTX_add_client_CA does not take ownership of the x509. It just calls X509_get_subject_name
             // and make a duplicate of this value. So we should always free the x509 after this call.
             // See https://github.com/netty/netty/issues/6249.
+#endif // OPENSSL_IS_BORINGSSL
+
             X509_free(x509);
             n++;
         }
+
+#ifdef OPENSSL_IS_BORINGSSL
+        SSL_CTX_set0_client_CAs(ctx, names);
+#endif // OPENSSL_IS_BORINGSSL
+
     } else {
+
+#ifdef OPENSSL_IS_BORINGSSL
+        return -1;
+#else
         /* free a perhaps already configured extra chain */
         SSL_CTX_clear_extra_chain_certs(ctx);
 
@@ -502,8 +512,9 @@ static int SSL_CTX_setup_certs(SSL_CTX *ctx, BIO *bio, bool skipfirst, bool ca)
             }
             n++;
         }
-    }
+#endif // OPENSSL_IS_BORINGSSL
 
+    }
     /* Make sure that only the error is just an EOF */
     if ((err = ERR_peek_error()) > 0) {
         if (!(   ERR_GET_LIB(err) == ERR_LIB_PEM
@@ -520,29 +531,22 @@ int tcn_SSL_CTX_use_certificate_chain_bio(SSL_CTX *ctx, BIO *bio, bool skipfirst
     return SSL_CTX_setup_certs(ctx, bio, skipfirst, false);
 }
 
-
 int tcn_SSL_CTX_use_client_CA_bio(SSL_CTX *ctx, BIO *bio)
 {
     return SSL_CTX_setup_certs(ctx, bio, false, true);
 }
 
+#ifndef OPENSSL_IS_BORINGSSL
 int tcn_SSL_use_certificate_chain_bio(SSL *ssl, BIO *bio, bool skipfirst)
 {
-#if defined(LIBRESSL_VERSION_NUMBER)
-    // Only supported on openssl 1.0.2+
+#if defined(LIBRESSL_VERSION_NUMBER) && LIBRESSL_VERSION_NUMBER <= 0x2090200fL
+    // Only supported on openssl 1.0.2+ and LibreSSL 2.9.2
     return -1;
 #else
-    X509 *x509;
+    X509 *x509 = NULL;
     unsigned long err;
     int n;
 
-#ifndef OPENSSL_IS_BORINGSSL
-    if (OpenSSL_version_num() < 0x10002000L) {
-        // Only supported on openssl 1.0.2+
-       return -1;
-    }
-#endif
-
     /* optionally skip a leading server certificate */
     if (skipfirst) {
         if ((x509 = PEM_read_bio_X509(bio, NULL, NULL, NULL)) == NULL) {
@@ -570,7 +574,7 @@ int tcn_SSL_use_certificate_chain_bio(SSL *ssl, BIO *bio, bool skipfirst)
         ERR_clear_error();
     }
     return n;
-#endif
+#endif // defined(LIBRESSL_VERSION_NUMBER)
 }
 
 X509 *tcn_load_pem_cert_bio(const char *password, const BIO *bio)
@@ -586,6 +590,7 @@ X509 *tcn_load_pem_cert_bio(const char *password, const BIO *bio)
     }
     return cert;
 }
+#endif // OPENSSL_IS_BORINGSSL
 
 EVP_PKEY *tcn_load_pem_key_bio(const char *password, const BIO *bio)
 {
@@ -660,9 +665,9 @@ int select_next_proto(SSL *ssl, const unsigned char **out, unsigned char *outlen
 
     unsigned int i = 0;
     unsigned char target_proto_len;
-    unsigned char *p;
-    const unsigned char *end;
-    unsigned char *proto;
+    unsigned char *p = NULL;
+    const unsigned char *end = NULL;
+    unsigned char *proto = NULL;
     unsigned char proto_len;
 
     while (i < supported_protos_len) {
