@@ -156,6 +156,11 @@ public final class SSL {
     public static final int SSL_ERROR_ZERO_RETURN      = sslErrorZeroReturn();
     public static final int SSL_ERROR_WANT_CONNECT     = sslErrorWantConnect();
     public static final int SSL_ERROR_WANT_ACCEPT      = sslErrorWantAccept();
+    // https://boringssl.googlesource.com/boringssl/+/chromium-stable/include/openssl/ssl.h#519
+    public static final int SSL_ERROR_WANT_PRIVATE_KEY_OPERATION = sslErrorWantPrivateKeyOperation();
+
+    // BoringSSL specific
+    public static final int SSL_ERROR_WANT_CERTIFICATE_VERIFY = sslErrorWantCertificateVerify();
 
     /**
      * SSL_new
@@ -604,7 +609,10 @@ public final class SSL {
      * @param bio BIO of PEM-encoded Server CA Certificates.
      * @param skipfirst Skip first certificate if chain file is inside
      *                  certificate file.
+     *
+     * @deprecated use {@link #setKeyMaterial(long, long, long)}
      */
+    @Deprecated
     public static native void setCertificateChainBio(long ssl, long bio, boolean skipfirst);
 
     /**
@@ -628,7 +636,10 @@ public final class SSL {
      * @param password Certificate password. If null and certificate
      *                 is encrypted.
      * @throws Exception if an error happened
+     *
+     * @deprecated use {@link #setKeyMaterial(long, long, long)}
      */
+    @Deprecated
     public static native void setCertificateBio(
             long ssl, long certBio, long keyBio, String password) throws Exception;
 
@@ -789,4 +800,43 @@ public final class SSL {
      * @return the signature algorithms or {@code null}.
      */
     public static native String[] getSigAlgs(long ssl);
+
+    /**
+     * Returns the master key used for the current ssl session.
+     * This should be used extremely sparingly as leaking this key defeats the whole purpose of encryption
+     * especially forward secrecy. This exists here strictly for debugging purposes.
+     *
+     * @param ssl the SSL instance (SSL *)
+     * @return the master key used for the ssl session
+     */
+    public static native byte[] getMasterKey(long ssl);
+
+    /**
+     * Extracts the random value sent from the server to the client during the initial SSL/TLS handshake.
+     * This is needed to extract the HMAC & keys from the master key according to the TLS PRF.
+     * <b>This is not a random number generator.</b>
+     *
+     * @param ssl the SSL instance (SSL *)
+     * @return the random server value used for the ssl session
+     */
+    public static native byte[] getServerRandom(long ssl);
+
+    /**
+     * Extracts the random value sent from the client to the server during the initial SSL/TLS handshake.
+     * This is needed to extract the HMAC & keys from the master key according to the TLS PRF.
+     * <b>This is not a random number generator.</b>
+     *
+     * @param ssl the SSL instance (SSL *)
+     * @return the random client value used for the ssl session
+     */
+    public static native byte[] getClientRandom(long ssl);
+
+    /**
+     * Return the {@link Runnable} thats needs to be run as an operation returned {@link #SSL_ERROR_WANT_X509_LOOKUP}.
+     * After the task was run we should retry the operations that returned {@link #SSL_ERROR_WANT_X509_LOOKUP}.
+     *
+     * @param ssl the SSL instance (SSL *)
+     * @return the task to run.
+     */
+    public static native Runnable getTask(long ssl);
 }
